/// <reference lib="webworker" />

import { Envelope, Syncable, User } from 'cic-client-meta';
import { add0x } from '@src/assets/js/ethtx/dist/hex';
import { personValidation, vcardValidation } from '@app/_helpers/schema-validation';
// const vCard = require('vcard-parser');
import * as vCard from 'vcard-parser';

const headers = {
  'x-cic-automerge': 'client',
};
const options = {
  headers,
};

addEventListener('message', async ({ data }) => {
  if (data.addresses instanceof Array) {
    for (const accountAddress of data.addresses) {
      const account = await getAccountByAddress(accountAddress, data.url);
      postMessage(account);
    }
  }
});

async function getAccountByAddress(accountAddress: string, metaUrl: string): Promise<any> {
  const userKey = await User.toKey(add0x(accountAddress));
  return new Promise((resolve, reject) => {
    fetch(`${metaUrl}/${userKey}`, options).then(async (response) => {
      if (!response.ok) {
        return reject({
          status: response.status,
          statusText: response.statusText,
        });
      } else {
        const account: Syncable = Envelope.fromJSON(JSON.stringify(response.text())).unwrap();
        const accountInfo = account.m.data;
        await personValidation(accountInfo);
        accountInfo.vcard = vCard.parse(atob(accountInfo.vcard));
        await vcardValidation(accountInfo.vcard);
        return resolve(accountInfo);
      }
    });
  });
}
